// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024-2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aoc/LanParty.hpp>

const std::string LanParty::getPuzzleName() const
{
    return "LAN Party";
}

const int LanParty::getPuzzleDay() const
{
    return 23;
}

void LanParty::processDataLine(const std::string& line)
{
    lan_.addEdge(findOrAddVertex(line.substr(0, 2)), findOrAddVertex(line.substr(3, 2)));
}

void LanParty::finish()
{
    auto itTx = labelMap_.lower_bound(getFirstTxComputerName());
    const auto itTxEnd = labelMap_.upper_bound(getLastTxComputerName());
    while (itTx != itTxEnd)
    {
        computeInterconnectedThreeSetCount(itTx->second);
        itTx++;
    }
}

constexpr std::string LanParty::getFirstTxComputerName()
{
    return "ta";
}

constexpr std::string LanParty::getLastTxComputerName()
{
    return "tz";
}

int LanParty::findOrAddVertex(const std::string& vertexId)
{
    const auto found = labelMap_.find(vertexId);
    if (found != labelMap_.end())
    {
        return found->second;
    }
    else
    {
        int vertex = lan_.addVertex(vertexId);
        labelMap_.insert({ vertexId, vertex });
        return vertex;
    }
}

void LanParty::computeInterconnectedThreeSetCount(const int vertexTx)
{
    auto itFirstNeighbor = lan_.begin(vertexTx);
    while (itFirstNeighbor != lan_.end())
    {
        if (canProcessVertex(itFirstNeighbor->vertex, vertexTx))
        {
            auto itSecondNeighbor = itFirstNeighbor;
            itSecondNeighbor++;
            while (itSecondNeighbor != lan_.end())
            {
                if (canProcessVertex(itSecondNeighbor->vertex, vertexTx))
                {
                    if (lan_.areAdjacent(itFirstNeighbor->vertex, itSecondNeighbor->vertex))
                    {
                        part1++;
                    }
                }
                itSecondNeighbor++;
            }
        }
        itFirstNeighbor++;
    }
}

bool LanParty::canProcessVertex(const int vertexToCheck, const int vertexTx) const
{
    const std::string& label{ lan_.getVertexData(vertexToCheck) };
    return (label[0] != 't' || label > lan_.getVertexData(vertexTx));
}
